//////////////////////////////////////////////////////////////////////
// aft_analog_logic_channes.cpp - Device Manager Property Page for
//			A200/A400 Analog adapters.
//
// Author	:	David Rokhvarg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdlacfg.h"

#define front_end_cfg wandev_conf.fe_cfg

////////////////////////////////////////////////////////////////////////////////
//AFT cfg functions prototypes

static int AFT_Analog_OnNotify(HWND    ParentHwnd,
				LPNMHDR NmHdr,
				SETUPSTUFF * Params
				);

static void AFT_Init(HKEY	hkey, HWND hdlg, GLOBALS* cfg);

static int display_analog_per_card_properties(int logic_ch_number, HWND hdlg);

//interface 0 will store the per-card configuration
#define MASTER_INTERFACE_NUMBER 0

static int selected_ind=0;
static LRESULT CALLBACK analog_per_card_properties_dlg(HWND hdlg, UINT iMessage,
										 WPARAM wParam, LPARAM lParam);

static int get_logic_channel_properties_data(HWND hdlg);
static BOOL aft_check_all_cfg(HWND hdlg);

static int DisplayContextMenu(HWND hdlg, POINT pt);
static HMENU hmenu;   // group menu 

///////////////////////////////////////////////////////////////////////////////
static char tmp_char_buffer[TMP_BUFFER_LEN];

///////////////////////////////////////////////////////////////////////////////
#define NUM_OF_OPER_MODE_OPTIONS	2

char* analog_operational_mode_options[NUM_OF_OPER_MODE_OPTIONS] = 
{	SDLA_DECODE_USEDBY_FIELD(TDM_CHAN_VOICE_API),
	SDLA_DECODE_USEDBY_FIELD(TDM_VOICE_API)
};

///////////////////////////////////////////////////////////////////////////////
static char* fxo_modes[] =
{
        "FCC",
        "TBR21",
        "ARGENTINA",
        "AUSTRALIA",
        "AUSTRIA",
        "BAHRAIN",
        "BELGIUM",
        "BRAZIL",
        "BULGARIA",
        "CANADA",
        "CHILE",
        "CHINA",
        "COLUMBIA",
        "CROATIA",
        "CYRPUS",
        "CZECH",
        "DENMARK",
        "ECUADOR",
        "EGYPT",
        "ELSALVADOR",
        "FINLAND",
        "FRANCE",
        "GERMANY",
        "GREECE",
        "GUAM",
        "HONGKONG",
        "HUNGARY",
        "ICELAND",
        "INDIA",
        "INDONESIA",
        "IRELAND",
        "ISRAEL",
        "ITALY",
        "JAPAN",
        "JORDAN",
        "KAZAKHSTAN",
        "KUWAIT",
        "LATVIA",
        "LEBANON",
        "LUXEMBOURG",
        "MACAO",
        "MALAYSIA",
        "MALTA",
        "MEXICO",
        "MOROCCO",
        "NETHERLANDS",
        "NEWZEALAND",
        "NIGERIA",
        "NORWAY",
        "OMAN",
        "PAKISTAN",
        "PERU",
        "PHILIPPINES",
        "POLAND",
        "PORTUGAL",
        "ROMANIA",
        "RUSSIA",
        "SAUDIARABIA",
        "SINGAPORE",
        "SLOVAKIA",
        "SLOVENIA",
        "SOUTHAFRICA",
        "SOUTHKOREA",
        "SPAIN",
        "SWEDEN",
        "SWITZERLAND",
        "SYRIA",
        "TAIWAN",
        "TAIWAN",
        "THAILAND",
        "UAE",
        "UK",
        "USA",
        "YEMEN",
        NULL
};

const look_up_t aft_analog_fxo_tapping_options[] = {
	{"Off",	WAN_RM_DEFAULT},
	{"On",	WAN_RM_TAPPING},
	{NULL,		0}
};

///////////////////////////////////////////////////////////////////////////////

DLGPROC_RETURN_TYPE WINAPI AFT_Analog_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	int i;

	switch (msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DBG_ANALOG_PAGE, "WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if(AFT_Analog_OnNotify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DBG_ANALOG_PAGE, "Invalid Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		{
#if 0		//June 4, 2009 : removed the only option is to start Wanpipemon because Wanpipemon
			//is not QT GUI anymore, but a portable "c" code, common with Linux.
			// Load the menu resource. 
			if ((hmenu = LoadMenu(hInst, MAKEINTRESOURCE(IDR_MENU_ANALOG_OPTIONS))) == NULL) 
			{	//
				Debug(DBG_ANALOG_PAGE, "hmenu == NULL!!\n");
			}
#endif
			if (okay == TRUE)
			{
				//initialize visual controls
				AFT_Init(hkey, hdlg, &Globals_New_Cfg);
			}

			if (!okay)
			{					// no SampleInfo
				ShowWindow(GetDlgItem(hdlg, IDC_INFOLABEL), SW_HIDE);
				ShowWindow(GetDlgItem(hdlg, IDB_MOREINFO), SW_HIDE);
			}					// no SampleInfo
		}
		break;
		
   	case WM_DESTROY:
		Debug(DBG_ANALOG_PAGE, "Samclass:WM_DESTROY");
		{
			LPSTR infofile = (LPSTR) GetWindowLongPtr(hdlg, DWL_USER);
			if (infofile){
				GlobalFree((HGLOBAL) infofile);
			}

			if(hmenu){
				// Destroy the menu. 
				DestroyMenu(hmenu); 
			}
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DBG_ANALOG_PAGE, "DBG_ANALOG_PAGE:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		AFT_Init(hkey, hdlg, &Globals_New_Cfg);
		break;
		
	case WM_CONTEXTMENU:
		Debug(DBG_ANALOG_PAGE, "WM_CONTEXTMENU\n");
		//make sure:
		//1. mouse above LIST_GROUP_PROPERTIES
		//
		//2. there is an active selection in LIST_GROUP_PROPERTIES
		//   before displaying the menu.
		{ 
			RECT	rc;                    // client area of window 
			int		x = GET_X_LPARAM(lParam);
			int		y = GET_Y_LPARAM(lParam);
			POINT	pt = { x, y };        // location of mouse click 

			// Get the bounding rectangle of the client area. 
			GetClientRect(GetDlgItem(hdlg, LIST_GROUP_PROPERTIES), &rc); 
 
			// Convert the mouse position to client coordinates. 
			ScreenToClient(GetDlgItem(hdlg, LIST_GROUP_PROPERTIES), &pt); 
				
			// If the position is in the client area, display a  
			// shortcut menu. 
			if (PtInRect(&rc, pt)) 
			{ 
				selected_ind = (int)SendDlgItemMessage(hdlg, LIST_GROUP_PROPERTIES,
					LB_GETCURSEL, 0, 0);
				if(selected_ind == LB_ERR){
					MessageBox(GetActiveWindow(), "No Group was selected", "",
						MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
					break;
				}

				ClientToScreen(GetDlgItem(hdlg, LIST_GROUP_PROPERTIES), &pt); 
				switch(DisplayContextMenu(GetDlgItem(hdlg, LIST_GROUP_PROPERTIES), pt))
				{
				case MENU_ITEM_MONITOR:
					if(is_legacy_api_mode(Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].operational_mode) == false){
						//Interface name e.g.: WANPIPE1_IF1 - if names start from 1
						start_monitor(selected_ind + 1);
					}else{
						start_monitor(selected_ind);
					}
					break;
				}//switch()
			}//if() 
		} 
		break;

    case WM_COMMAND:
		switch (LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(), "WanpipePnp.hlp", HELP_CONTEXT,
				IDH_LINE_CFG);
			break;


		case COMBO_NUMBER_OF_GROUPS:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELCHANGE){
				//Number of groups changed, ask user to confirm,
				//cause it is a serious change.
				if(yes_no_question("Are you sure you want change the number of groups?")
					== IDYES){
		
					i = SendDlgItemMessage(hdlg, COMBO_NUMBER_OF_GROUPS, CB_GETCURSEL, 0, 0);
					Globals_New_Cfg.aft_number_logic_channels = i + 1;

					//reinitialize visual controls with updated cfg
					AFT_Init(hkey, hdlg, &Globals_New_Cfg);
				}else{
					//make the current one selected
					SendDlgItemMessage(hdlg, COMBO_NUMBER_OF_GROUPS, CB_SETCURSEL,
						Globals_New_Cfg.aft_number_logic_channels - 1, 0);
				}
			}
			break;

		case BUTTON_MISC_INTERFACE_OPTIONS:
			Debug(DBG_ANALOG_PAGE, "BUTTON_MISC_INTERFACE_OPTIONS pressed\n");
			display_analog_per_card_properties(selected_ind, hdlg);
			//re-initialize visual controls
			AFT_Init(hkey, hdlg, &Globals_New_Cfg);
			break;
		}					// process command notification
		return TRUE;
	}						// process message

	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

static int DisplayContextMenu(HWND hdlg, POINT pt) 
{ 
	HMENU hmenuTrackPopup;  // shortcut menu 
	int menu_item_id;

	if(!hmenu){
		return 0;
	}

	// TrackPopupMenu cannot display the menu bar so get 
	// a handle to the first shortcut menu. 
	hmenuTrackPopup = GetSubMenu(hmenu, 0); 
 
	// Display the shortcut menu. Track both mouse buttons. 
	menu_item_id = TrackPopupMenu(	hmenuTrackPopup, 
							TPM_LEFTALIGN | TPM_RIGHTBUTTON | TPM_RETURNCMD, 
							pt.x,
							pt.y,
							0,
							hdlg, NULL); 

	Debug(DBG_ANALOG_PAGE, "menu_item_id: %d\n", menu_item_id);

	return menu_item_id;
} 

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK AFT_Analog_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DBG_ANALOG_PAGE, "PSPCB_RELEASE : AFT_Analog_PageCallbackProc()");
		RegCloseKey(hkey);

		//compare the configuration structures, restart if different.
		if((released_by_OK_button == 1) &&
		   memcmp(&Globals_Old_Cfg, &Globals_New_Cfg, sizeof(Globals_New_Cfg)))
		{
			if(yes_no_question("Device Configuration was changed. Do you want the Device to be restarted now?")
					== IDYES){
				restart_device();
			}else{
				MessageBox(GetActiveWindow(), (LPCTSTR)"Device was NOT restarted.",
					"Note for the user", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
			}
		}
		delete (SETUPSTUFF*) p->lParam;
		return 0;//return code is ignored
	}
	return TRUE;	// matters only for PSPCB_CREATE
}

///////////////////////////////////////////////////////////////////////////////

static int AFT_Analog_OnNotify(	HWND		ParentHwnd,
								LPNMHDR		NmHdr,
								SETUPSTUFF	*Params	)
{
    SP_DEVINSTALL_PARAMS spDevInstall = {0};
    TCHAR                friendlyName[LINE_LEN] ={0};
	BOOL	configuration_check_passed = FALSE;

	Debug(DBG_ANALOG_PAGE, "AFT_Analog_OnNotify()\n");
		
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = aft_check_all_cfg(ParentHwnd);

		Debug(DBG_ANALOG_PAGE, "AFT_Analog_OnNotify((): PSN_APPLY");

		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE){
			return HANDLE_OK;
		}else{
			return HANDLE_ERROR;
		}
		break;

	case PSN_QUERYCANCEL:
		Debug(DBG_ANALOG_PAGE, "AFT_Analog_OnNotify(() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
    default:
		Debug(DBG_ANALOG_PAGE, "AFT_Analog_OnNotify(() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
}

///////////////////////////////////////////////////////////////////////////////

static BOOL aft_check_all_cfg(HWND hdlg)
{	
	int i;
	unsigned long tmp_active_ch = 0;
	unsigned long cumulutive_active_ch = 0;
	static char local_tmp_buff[TMP_BUFFER_LEN];
	static char local_tmp_buff_1[TMP_BUFFER_LEN];

	Debug(DBG_ANALOG_PAGE, "aft_check_all_cfg()");
	
	Debug(DBG_ANALOG_PAGE, "*********  Writing Analog Configuration  ********\n");

	//the check passed, save to registry
	SetRegistryStringValue(hkey, "aft_number_of_logic_channels",
		Globals_New_Cfg.aft_number_logic_channels);

	for(i = 0; i < Globals_New_Cfg.aft_number_logic_channels; i++)
	{	//active channels
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_active_ch", i);
		SetRegistryStringValue(hkey,  local_tmp_buff,
			(char*)Globals_New_Cfg.aft_logic_ch_cfg[i].active_ch_str);

		//line mode
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_line_mode", i);
		SetRegistryStringValue(hkey, local_tmp_buff,
			(char*)Globals_New_Cfg.aft_logic_ch_cfg[i].line_mode);

		///////////////////////////////////////////////////////////////////////
		//MTU - the same in 0, for ALL!
		if(i != MASTER_INTERFACE_NUMBER){
			//do NOT copy buffer to itselt, cause it's an error!!
			snprintf(	Globals_New_Cfg.aft_logic_ch_cfg[i].mtu, TMP_BUFFER_LEN,
						"%s",
						Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].mtu);
		}

		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_mtu", i);
		SetRegistryStringValue(hkey, local_tmp_buff,
			(char*)Globals_New_Cfg.aft_logic_ch_cfg[i].mtu);

		///////////////////////////////////////////////////////////////////////
		//operation mode
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_operational_mode", i);
		SetRegistryStringValue(hkey, local_tmp_buff,
			(char*)Globals_New_Cfg.aft_logic_ch_cfg[i].operational_mode);

		///////////////////////////////////////////////////////////////////////
		//Idle Char - the same in 0, for ALL!
		Globals_New_Cfg.aft_logic_ch_cfg[i].decimal_idle_char = 
			Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].decimal_idle_char;

		//Idle char. Write in DECIMAL !!!
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "aft_logic_channel_%d_idle_char", i);
	
		_itoa(Globals_New_Cfg.aft_logic_ch_cfg[i].decimal_idle_char, local_tmp_buff_1, 10);

		SetRegistryStringValue(hkey, local_tmp_buff, local_tmp_buff_1);
	}

	//Analog global settings
	SetRegistryStringValue(hkey, "remora_fxo_operation_mode_name", Globals_New_Cfg.front_end_cfg.cfg.remora.opermode_name);

	SetRegistryStringValue(hkey, "RM_BATTTHRESH",	Globals_New_Cfg.front_end_cfg.cfg.remora.battthresh);

	SetRegistryStringValue(hkey, "RM_BATTDEBOUNCE", Globals_New_Cfg.front_end_cfg.cfg.remora.battdebounce);
	
	SetRegistryStringValue(hkey, "RM_FXO_TAPPING",	Globals_New_Cfg.front_end_cfg.cfg.remora.rm_mode);

	SetRegistryStringValue(hkey, "RM_FXO_TAPPING_OFF_HOOK_THRESHOLD",	Globals_New_Cfg.front_end_cfg.cfg.remora.ohthresh);

	Debug(DBG_ANALOG_PAGE, "*********  Finished writing Analog Configuration  ********\n");
	///////////////////////////////////////////////////////////////////

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////

static void AFT_Init(HKEY	hkey, HWND hdlg, GLOBALS* cfg)
{
	int i;
	static char local_tmp_buff[TMP_BUFFER_LEN];

	//clear before adding
	SendMessage(GetDlgItem(hdlg, LIST_GROUP_PROPERTIES), LB_RESETCONTENT, 0, 0); 

	//number of groups may change so remove all and redraw.
	SendDlgItemMessage(hdlg, COMBO_NUMBER_OF_GROUPS, CB_RESETCONTENT, 0, 0);

	//add strings to Number of groups combobox
	for(i = 0; i < NUM_OF_E1_TIMESLOTS; i++){
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", i + 1);
		SendDlgItemMessage(hdlg, COMBO_NUMBER_OF_GROUPS, CB_ADDSTRING, 0,
			(LPARAM)local_tmp_buff);
	}
	//make the current one selected
	SendDlgItemMessage(hdlg, COMBO_NUMBER_OF_GROUPS, CB_SETCURSEL,
		cfg->aft_number_logic_channels - 1, 0);

	Debug(DBG_ANALOG_PAGE, "analog: cfg->aft_number_logic_channels: %d\n", 
		cfg->aft_number_logic_channels);

	for(i = 0; i < cfg->aft_number_logic_channels; i++){
		if(is_legacy_api_mode(Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].operational_mode) == false){
			//Interface name e.g.: WANPIPE1_IF1 - if names start from 1
			_snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%s_IF%d\t\t\t", Globals_New_Cfg.DeviceName, i + 1);
		}else{
			//Interface name e.g.: WANPIPE1_IF0 - if names start from 0
			_snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%s_IF%d\t\t\t", Globals_New_Cfg.DeviceName, i);
		}

		//Module type: FXO/FXS
		_snprintf(&local_tmp_buff[strlen(local_tmp_buff)],
			TMP_BUFFER_LEN-strlen(local_tmp_buff),
			"%5s  ",
			(cfg->aft_logic_ch_cfg[i].analog_module_type == MOD_TYPE_FXO ? "FXO" :
			(cfg->aft_logic_ch_cfg[i].analog_module_type == MOD_TYPE_FXS ? "FXS" : "?")));

		//display it all
		SendMessage(GetDlgItem(hdlg, LIST_GROUP_PROPERTIES), LB_ADDSTRING, 0, (LPARAM)local_tmp_buff); 
	}
}

///////////////////////////////////////////////////////////////////////////////

static int display_analog_per_card_properties(int logic_ch_number, HWND hdlg)
{
	INT_PTR rval;

	rval = DialogBox(hInst, MAKEINTRESOURCE(AFT_ANALOG_CHAN_PROPERTIES), hdlg, 
		(DLGPROC)analog_per_card_properties_dlg);

	if(rval == -1){
		LPVOID lpMsgBuf;
		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);
		return FALSE;
	}
	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////

static LRESULT CALLBACK analog_per_card_properties_dlg(HWND hdlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	int i, current_selection_index;
	static char local_tmp_buff[TMP_BUFFER_LEN];
	
	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;

	case WM_INITDIALOG:

		Debug(DBG_ANALOG_PAGE, "analog_per_card_properties_dlg(): WM_INITDIALOG\n");
		
		//////////////////////////////////////////////////////////////////////
		//set dialog's title		
		_snprintf(local_tmp_buff, TMP_BUFFER_LEN, "Properties of: %s", 
			Globals_New_Cfg.DeviceName);
		SetWindowText(hdlg, local_tmp_buff);

		///////////////////////////////////////////////////////////////////////
		//set the MTU value
		SetDlgItemText(hdlg, EDIT_MTU, Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].mtu);

		///////////////////////////////////////////////////////////////////////
		//add strings to API Operational Mode combobox
		SendDlgItemMessage(hdlg, API_COMBO_OPERATIONAL_MODE, CB_RESETCONTENT, 0, 0);
		for(i = 0; i < NUM_OF_OPER_MODE_OPTIONS; i++){
			SendDlgItemMessage(hdlg, API_COMBO_OPERATIONAL_MODE, CB_ADDSTRING, 0,
				(LPARAM)analog_operational_mode_options[i]);
		}

		//make the current one selected
		i = SendDlgItemMessage(	hdlg, API_COMBO_OPERATIONAL_MODE, CB_FINDSTRING, -1,
								(LPARAM)Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].operational_mode);
		
		SendDlgItemMessage(hdlg, API_COMBO_OPERATIONAL_MODE, CB_SETCURSEL, (i != CB_ERR ? i : 0), 0);

		///////////////////////////////////////////////////////////////////////
		//Add strings to FXO Operational Mode combobox and find the index of 
		//current setting.
		current_selection_index = 0;
		SendDlgItemMessage(hdlg, COMBO_FXO_OPER_MODE, CB_RESETCONTENT, 0, 0);
		for(i = 0; fxo_modes[i] != NULL; i++){
			SendDlgItemMessage(hdlg, COMBO_FXO_OPER_MODE, CB_ADDSTRING, 0, (LPARAM)fxo_modes[i]);
			if(_stricmp(fxo_modes[i], Globals_Old_Cfg.front_end_cfg.cfg.remora.opermode_name) == 0){
				current_selection_index = i;
			}
		}

		//make the current one selected
		SendDlgItemMessage(hdlg, COMBO_FXO_OPER_MODE, CB_SETCURSEL, current_selection_index, 0);

		///////////////////////////////////////////////////////////////////////
		//Show Idle char value in HEX!!!
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%02X", 
			Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].decimal_idle_char);

		SetDlgItemText(hdlg, EDIT_IDLE_CHAR, local_tmp_buff);

		///////////////////////////////////////////////////////////////////////
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.front_end_cfg.cfg.remora.battthresh);
		SetDlgItemText(hdlg, EDIT_BATTERY_THRESHOLD, local_tmp_buff);

		///////////////////////////////////////////////////////////////////////
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.front_end_cfg.cfg.remora.battdebounce);
		SetDlgItemText(hdlg, EDIT_BATTERY_DEBOUNCE,	local_tmp_buff);

		///////////////////////////////////////////////////////////////////////
		SendDlgItemMessage(hdlg, COMBO_FXO_TAPPING, CB_RESETCONTENT, 0, 0);
		//add the lines to combobox
		i=0; current_selection_index = 0;
		while(aft_analog_fxo_tapping_options[i].name != NULL)
		{	SendDlgItemMessage(	hdlg, COMBO_FXO_TAPPING, CB_ADDSTRING, 0, (LPARAM)aft_analog_fxo_tapping_options[i].name);
			if(aft_analog_fxo_tapping_options[i].value == Globals_New_Cfg.front_end_cfg.cfg.remora.rm_mode){
				current_selection_index = i;
			}
			i++;
		}
		//make the current one selected
		SendDlgItemMessage(hdlg, COMBO_FXO_TAPPING, CB_SETCURSEL, current_selection_index, 0);

		///////////////////////////////////////////////////////////////////////
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.front_end_cfg.cfg.remora.ohthresh);
		SetDlgItemText(hdlg, EDIT_OFFHOOK_THRESHOLD, local_tmp_buff);

		return(TRUE);

	case WM_COMMAND:

		switch(GET_WM_COMMAND_ID(wParam, lParam)) 
		{
		case IDOK:
			if(get_logic_channel_properties_data(hdlg) == TRUE){
				EndDialog(hdlg, 0);
			}
			break;

		case IDCANCEL:
			EndDialog(hdlg, 0);
			//user cancelled any changes, restore original values for the group
			memcpy(&Globals_New_Cfg, &Globals_Old_Cfg, sizeof(GLOBALS));
			break;
		}
	}
	return(0);
}

//////////////////////////////////////////////////////////////////////////////

static int get_logic_channel_properties_data(HWND hdlg)
{
	int i, rc, max_mtu;
	static char local_tmp_buff[TMP_BUFFER_LEN];

	/////////////////////////////////////////////////////////////////////////
	//get operational mode
	i = SendDlgItemMessage(hdlg, API_COMBO_OPERATIONAL_MODE, CB_GETCURSEL, 0, 0);

	Debug(DBG_ANALOG_PAGE, "orig oper mode: %s\n",
		Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].operational_mode);

	strcpy(Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].operational_mode,
		analog_operational_mode_options[i]);

	Debug(DBG_ANALOG_PAGE, "new oper mode: %s\n",
		Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].operational_mode);


	/////////////////////////////////////////////////////////////////////////
	//get MTU

	Debug(DBG_ANALOG_PAGE, "orig mtu: %d\n",
		Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].mtu);

	GetDlgItemText(hdlg, EDIT_MTU,
		Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].mtu,
		TMP_BUFFER_LEN);

	Debug(DBG_ANALOG_PAGE, "new mtu: %d\n",
		Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].mtu);

	i = atoi(Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].mtu);
	max_mtu = MAX_AFT_MTU;

	if(i < MIN_AFT_MTU || i > max_mtu){

		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"MTU\". Minimum is %d, Maximum is %d.",
			MIN_AFT_MTU, max_mtu);

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error",
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	if(i % 4){
		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"MTU\". Must be divizible by 4.");

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error",
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	/////////////////////////////////////////////////////////////////////////
	//get Idle char
	GetDlgItemText(hdlg, EDIT_IDLE_CHAR, local_tmp_buff, TMP_BUFFER_LEN);

	rc = sscanf(local_tmp_buff, "%X", &i);
	if(rc == 0 || rc == EOF){
		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"Idle char\". Has to be a Hexadecimal char (i.g.: 7E).");

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error", //szTitleString,
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}
	Globals_New_Cfg.aft_logic_ch_cfg[MASTER_INTERFACE_NUMBER].decimal_idle_char = (unsigned char)i;
	Debug(DBG_ANALOG_PAGE, "idle_char: 0x%02X (decimal: %d)\n", i, i);

	/////////////////////////////////////////////////////////////////////////
	//get operational mode name (used only for fxo - FCC...)
	Debug(DBG_ANALOG_PAGE, "orig remora.opermode_name: %s\n", Globals_New_Cfg.front_end_cfg.cfg.remora.opermode_name);
	i = SendDlgItemMessage(hdlg, COMBO_FXO_OPER_MODE, CB_GETCURSEL, 0, 0);
	SendDlgItemMessage(hdlg, COMBO_FXO_OPER_MODE, CB_GETLBTEXT, i, (LPARAM)Globals_New_Cfg.front_end_cfg.cfg.remora.opermode_name);
	Debug(DBG_ANALOG_PAGE, "new remora.opermode_name: %s\n", Globals_New_Cfg.front_end_cfg.cfg.remora.opermode_name);

	/////////////////////////////////////////////////////////////////////////
	GetDlgItemText(hdlg, EDIT_BATTERY_THRESHOLD, local_tmp_buff, TMP_BUFFER_LEN);
	Globals_New_Cfg.front_end_cfg.cfg.remora.battthresh = atoi(local_tmp_buff);

	/////////////////////////////////////////////////////////////////////////
	GetDlgItemText(hdlg, EDIT_BATTERY_DEBOUNCE, local_tmp_buff, TMP_BUFFER_LEN);
	Globals_New_Cfg.front_end_cfg.cfg.remora.battdebounce = atoi(local_tmp_buff);

	/////////////////////////////////////////////////////////////////////////
	i = SendDlgItemMessage(hdlg, COMBO_FXO_TAPPING, CB_GETCURSEL, 0, 0);
	Globals_New_Cfg.front_end_cfg.cfg.remora.rm_mode = aft_analog_fxo_tapping_options[i].value;

	/////////////////////////////////////////////////////////////////////////
	GetDlgItemText(hdlg, EDIT_OFFHOOK_THRESHOLD, local_tmp_buff, TMP_BUFFER_LEN);
	if(atoi(local_tmp_buff) < MIN_OFFHOOK_THRESHOLD || atoi(local_tmp_buff) > MAX_OFFHOOK_THRESHOLD){
		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"Off Hook Threshold\". Has to be between %d and %d.",
			MIN_OFFHOOK_THRESHOLD, MAX_OFFHOOK_THRESHOLD);

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error", //szTitleString,
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	Globals_New_Cfg.front_end_cfg.cfg.remora.ohthresh = atoi(local_tmp_buff);

	return TRUE;
}

//////////////////////////////////////////////////////////////////////////////
